SystemOrganization addCategory: #'Magritte-Roe'!

!MADescription methodsFor: '*magritte-roe' stamp: 'lr 6/12/2007 23:48'!
sqlColumnName
	^ self propertyAt: #sqlColumnName ifAbsent: [ nil ]! !

!MADescription methodsFor: '*magritte-roe' stamp: 'lr 6/12/2007 23:48'!
sqlColumnName: aString
	self propertyAt: #sqlColumnName put: aString! !

!MADescription methodsFor: '*magritte-roe' stamp: 'lr 6/12/2007 23:49'!
sqlColumnType
	^ self propertyAt: #sqlColumnType ifAbsent: [ nil ]! !

!MADescription methodsFor: '*magritte-roe' stamp: 'lr 6/12/2007 23:49'!
sqlColumnType: aString
	self propertyAt: #sqlColumnType put: aString! !

Object subclass: #MAConnectionPool
	instanceVariableNames: 'relations relationsMutex connections connectionMonitor'
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'Magritte-Roe'!

!MAConnectionPool class methodsFor: 'initialization' stamp: 'lr 6/12/2007 23:25'!
initialize
	Smalltalk addToStartUpList: self; addToShutDownList: self.
	self startUp! !

!MAConnectionPool class methodsFor: 'accessing' stamp: 'lr 6/12/2007 23:25'!
instance
	^ Instance! !

!MAConnectionPool class methodsFor: 'initialization' stamp: 'lr 6/12/2007 23:24'!
shutDown
	Instance ifNotNil: [ Instance finalize ].
	Instance := nil! !

!MAConnectionPool class methodsFor: 'initialization' stamp: 'lr 6/13/2007 00:06'!
startUp
	Instance ifNotNil: [ self shutDown ].
	Instance := self new initialize: 10! !

!MAConnectionPool methodsFor: 'public' stamp: 'lr 6/12/2007 23:08'!
execute: aString
	^ self execute: aString ifError: [ :error | self error: error value ]! !

!MAConnectionPool methodsFor: 'public' stamp: 'lr 6/12/2007 23:11'!
execute: aString ifError: aBlock 
	| connection result |
	connection := self next.
	result := [ connection execute: aString ]
		ensure: [ self nextPut: connection ].
	^ result errorResponse isNil
		ifTrue: [ result rows ]
		ifFalse: [ aBlock value: result errorResponse ]! !

!MAConnectionPool methodsFor: 'initialization' stamp: 'lr 6/12/2007 23:18'!
finalize
	connections do: [ :connection |
		connection isConnected
			ifTrue: [ connection terminate ] ].
	connections := relations := nil! !

!MAConnectionPool methodsFor: 'initialization' stamp: 'lr 6/12/2007 23:19'!
initialize: anInteger
	relations := Dictionary new.
	relationsMutex := Semaphore forMutualExclusion.
	connections := OrderedCollection new: anInteger.
	connectionMonitor := Monitor new.
	anInteger timesRepeat: [ connections add: self newConnection ]! !

!MAConnectionPool methodsFor: 'private' stamp: 'lr 6/12/2007 23:22'!
newConnection
	^ PGConnection new startup; yourself! !

!MAConnectionPool methodsFor: 'private' stamp: 'lr 6/12/2007 23:18'!
next
	"Answer a the next connection that works."
	
	| connection |
	connection := connectionMonitor critical: [ 
		connectionMonitor waitWhile: [ connections isEmpty ].
		connections removeLast ].
	(connection isConnected not or: [ connection state ~= #ReadyForQuery ]) ifTrue: [
		connection isConnected 
			ifTrue: [ connection terminate ].
		^ self newConnection ].
	^ connection! !

!MAConnectionPool methodsFor: 'private' stamp: 'lr 6/12/2007 23:15'!
nextPut: aConnection
	connectionMonitor critical: [
		connections addLast: aConnection.
		connectionMonitor signalAll ]! !

!MAConnectionPool methodsFor: 'accessing' stamp: 'lr 6/13/2007 20:57'!
relationNamed: aString
	^ relationsMutex critical: [ relations at: aString ifAbsentPut: [ MAMagritteRelation name: aString connection: self ] ]! !

Object subclass: #MAMappedObject
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Roe'!

!MAMappedObject class methodsFor: 'public' stamp: 'lr 6/13/2007 00:18'!
create
	self relation create.
	self description do: [ :each | 
		(each sqlColumnName notNil and: [ each sqlColumnType notNil ]) ifTrue: [
			self relation  
				addColumn: each sqlColumnName 
				withType: each sqlColumnType ] ]! !

!MAMappedObject class methodsFor: 'accessing-description' stamp: 'lr 6/13/2007 20:51'!
descriptionId
	^ MANumberDescription new
		accessor: (MASelectorAccessor read: #id write: #privateId:);
		sqlColumnType: 'SERIAL PRIMARY KEY';
		sqlColumnName: 'id';
		beHidden;
		yourself! !

!MAMappedObject class methodsFor: 'public' stamp: 'lr 6/13/2007 00:07'!
drop
	self relation drop! !

!MAMappedObject class methodsFor: 'persistency' stamp: 'lr 6/13/2007 20:52'!
load: aRelation
	^ aRelation asSingleMappedCollection asArray! !

!MAMappedObject class methodsFor: 'persistency' stamp: 'lr 6/13/2007 00:07'!
loadAll
	^ self load: self relation! !

!MAMappedObject class methodsFor: 'persistency' stamp: 'lr 6/18/2007 21:02'!
loadId: anInteger
	| array |
	array := self load: (self relation 
		select: [ :each | each id = anInteger ]).
	^ array isEmpty
		ifFalse: [ ^ array first ]! !

!MAMappedObject class methodsFor: 'accessing' stamp: 'lr 6/13/2007 00:07'!
relation
	^ (MAConnectionPool instance relationNamed: self tableName) objectClass: self! !

!MAMappedObject class methodsFor: 'accessing' stamp: 'lr 6/13/2007 00:07'!
tableName
	^ self subclassResponsibility! !

!MAMappedObject methodsFor: 'testing' stamp: 'lr 6/13/2007 20:12'!
hasId
	^ id notNil! !

!MAMappedObject methodsFor: 'accessing' stamp: 'lr 6/13/2007 00:06'!
id
	^ id! !

!MAMappedObject methodsFor: 'private-sql' stamp: 'lr 6/13/2007 20:43'!
insert
	| descriptions result |
	descriptions := self mappedDescription children
		copyWithout: self class descriptionId.
	result := self relation basicQuery: (self relation
		sqlInsert: (descriptions collect: [ :each | each sqlColumnName  ])
		values: (descriptions collect: [ :each | self readUsing: each ])) , ';' ,
		(self relation sqlCurrentValue: self class descriptionId sqlColumnName).
	id := result first first! !

!MAMappedObject methodsFor: 'private' stamp: 'lr 6/13/2007 20:43'!
mappedDescription
	^ self description select: [ :each | each sqlColumnName notNil and: [ each sqlColumnType notNil ] ]! !

!MAMappedObject methodsFor: 'printing' stamp: 'lr 6/18/2007 20:48'!
printOn: aStream
	super printOn: aStream.
	self hasId
		ifTrue: [ aStream nextPut: $[; print: self id; nextPut: $] ]! !

!MAMappedObject methodsFor: 'private' stamp: 'lr 6/13/2007 20:50'!
privateId: anInteger
	id := anInteger! !

!MAMappedObject methodsFor: 'accessing' stamp: 'lr 6/13/2007 20:20'!
relation
	^ self class relation! !

!MAMappedObject methodsFor: 'public' stamp: 'lr 6/13/2007 20:20'!
save
	self hasId ifTrue: [ self update ] ifFalse: [ self insert ]! !

!MAMappedObject methodsFor: 'private-sql' stamp: 'lr 6/13/2007 20:42'!
update
	| descriptions result |
	descriptions := self mappedDescription children.
	result := self relation basicQuery: (self relation
		sqlInsert: (descriptions collect: [ :each | each sqlColumnName  ])
		values: (descriptions collect: [ :each | self readUsing: each ])) , ';' ,
		(self relation sqlCurrentValue: self class descriptionId sqlColumnName).
	id := result first first! !

!PGConnection class methodsFor: '*magritte-roe-overrides' stamp: 'yj 2/1/2006 11:37'!
buildDefaultFieldConverters
	"The type oid's can be found using:
		SELECT oid,typname from pg_type
	Also, see the source code file:
		.../src/include/catalog/pg_type.h
	"
	| converters dateBlock timestampBlock timetzBlock timestamptzBlock |

	converters := IdentityDictionary new.

	#(16 "bool" 1000 "_bool")
		do: [:each | converters at: each put: [:s | s = 't' or: [s = 'T']]].

	#(18 "char" 1002 "_char" 1042 "bpchar")
		do: [:each | converters at: each put: [:s | s]].

	#(20 "int8" 21 "int2" 23 "int4" 1005 "_int2" 1007 "_int4" 1016 "_int8")
		do: [:each | converters at: each put: [:s | Number readFrom: (ReadStream on: s)]].

	#(700 "float4" 701 "float8" 1021 "_float4" 1022 "_float8" 1700 "numeric")
		do: [:each | converters at: each put: [:s | (Number readFrom: (ReadStream on: s)) asFloat]].

	dateBlock :=
		[:s | Date
				newDay: (s copyFrom: 9 to: 10) asInteger
				month: (s copyFrom: 6 to: 7) asInteger
				year: (s copyFrom: 1 to: 4) asInteger].
	#(1082 "date" 1182 "_date")
		do: [:each | converters at: each put: dateBlock].

	#(1083 "time" 1183 "_time")
		do: [:each | converters at: each put: [:s | Time readFrom: (ReadStream on: s)]].

	timestampBlock :=
		[:s |	DateAndTime
				year: (s copyFrom: 1 to: 4) asInteger
				month: (s copyFrom: 6 to: 7) asInteger
				day: (s copyFrom: 9 to: 10) asInteger
				hour: (s copyFrom: 12 to: 13) asInteger
				minute: (s copyFrom: 15 to: 16) asInteger
				second: (s copyFrom: 18 to: 19) asInteger].
	#(1114 "timestamp" 1115 "_timestamp")
		do: [:each | converters at: each put: timestampBlock].

	timetzBlock :=
		[:s | | list |
			list := (s subStrings: ':') collect:[:i | i asInteger].
			Time fromSeconds: ((list at: 1) * 3600) + ((list at: 2) * 60) + (list at: 3)].
	#(1266 "timetz" 1270 "_timetz")
		do: [:each | converters at: each put: timetzBlock].

	timestamptzBlock :=
		[:s | | direction offset |
			direction := (s charAt: 23) = $- ifTrue: [-1] ifFalse: [1].
			offset := Duration
				days: 0
				hours: (s copyFrom: 21 to: 22) asInteger * direction
				minutes: (s copyFrom: 24 to: 25) asInteger * direction
				seconds: 0.
			DateAndTime
				year: (s copyFrom: 1 to: 4) asInteger
				month: (s copyFrom: 6 to: 7) asInteger
				day: (s copyFrom: 9 to: 10) asInteger
				hour: (s copyFrom: 12 to: 13) asInteger
				minute: (s copyFrom: 15 to: 16) asInteger
				second: (s copyFrom: 18 to: 19) asInteger
				offset: offset].
	#(1184 "timestamptz " 1185 "_timestamptz")
		do: [:each | converters at: each put: timestamptzBlock].

	^ converters
! !

RASQLRelation subclass: #MAMagritteRelation
	instanceVariableNames: 'objectClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Roe'!

!MAMagritteRelation methodsFor: 'public' stamp: 'lr 6/13/2007 00:13'!
addColumn: aString withType: aTypeString
	connection execute: (self sqlAddColumn: aString with: aTypeString).
	attributes := nil! !

!MAMagritteRelation methodsFor: 'public' stamp: 'lr 6/13/2007 20:11'!
addValues: anArray attributes: anAttributeArray
	self exec: (self sqlInsert: anAttributeArray values: anArray)! !

!MAMagritteRelation methodsFor: 'private' stamp: 'lr 6/12/2007 23:27'!
basicExec: aString
	^ connection execute: aString! !

!MAMagritteRelation methodsFor: 'private' stamp: 'lr 6/13/2007 00:19'!
basicQuery: aString
	^ (self basicExec: aString) resultSets rows collect: [ :each | each data ]! !

!MAMagritteRelation methodsFor: 'public' stamp: 'lr 6/13/2007 00:09'!
create
	connection execute: self sqlCreate.
	attributes := nil! !

!MAMagritteRelation methodsFor: 'private' stamp: 'lr 6/12/2007 23:30'!
discoverAttributes
	^ (self query: self sqlDiscoverAttributes) collect: [ :each | RASimpleAttribute named: each first relation: self ]! !

!MAMagritteRelation methodsFor: 'public' stamp: 'lr 6/13/2007 00:10'!
drop
	connection execute: self sqlDrop.
	attributes := nil! !

!MAMagritteRelation methodsFor: 'accessing' stamp: 'lr 6/12/2007 23:42'!
objectClass
	^ objectClass! !

!MAMagritteRelation methodsFor: 'accessing' stamp: 'lr 6/12/2007 23:47'!
objectClass: aClass
	objectClass := aClass! !

!MAMagritteRelation methodsFor: 'private' stamp: 'lr 6/21/2007 23:21'!
objectForValues: aDictionary
	| object description |
	object := self objectClass new.
	aDictionary valuesDo: [ :each |
		description := object description
			detect: [ :desc | desc sqlColumnName = each key name ]
			ifNone: [ nil ].
		description isNil ifFalse: [ 
			object 
				write: each value
				using: description ] ].
	^ object! !

!MAMagritteRelation methodsFor: 'public' stamp: 'lr 6/12/2007 23:28'!
size
	^ (self query: self sqlCount) first first! !

!MAMagritteRelation methodsFor: 'sql' stamp: 'lr 6/13/2007 00:24'!
sqlAddColumn: aString with: aTypeString
	^ 'ALTER TABLE ' , self name , ' ADD COLUMN "' , aString , '" ' , aTypeString! !

!MAMagritteRelation methodsFor: 'sql' stamp: 'lr 6/13/2007 00:16'!
sqlAttributes
	^ 'SELECT attname FROM pg_class, pg_attribute WHERE relname = ' , self name printString , ' AND attnum > 0 AND attrelid = pg_class.oid AND NOT attisdropped'! !

!MAMagritteRelation methodsFor: 'sql' stamp: 'lr 6/13/2007 00:16'!
sqlCreate
	^ 'CREATE TABLE ' , self name , ' ()'! !

!MAMagritteRelation methodsFor: 'sql' stamp: 'lr 6/13/2007 20:37'!
sqlCurrentValue: aString
	^ 'SELECT currval('  ,  (self name , '_' , aString , '_seq') printString , ')'! !

!MAMagritteRelation methodsFor: 'private' stamp: 'lr 6/12/2007 23:28'!
sqlDiscoverAttributes
	^ 'SELECT attname FROM pg_class, pg_attribute WHERE relname = ' , self name printString , ' AND attnum > 0 AND attrelid = pg_class.oid AND NOT attisdropped'! !

!MAMagritteRelation methodsFor: 'sql' stamp: 'lr 6/13/2007 00:16'!
sqlDrop
	^ 'DROP TABLE ' , self name! !

!MAMagritteRelation methodsFor: 'sql' stamp: 'lr 6/13/2007 00:16'!
sqlVacuum
	^ 'VACUUM ' , self name! !

!MAMagritteRelation methodsFor: 'public' stamp: 'lr 6/13/2007 00:10'!
vacuum
	connection execute: self sqlVacuum! !
MAConnectionPool initialize!
